//
//  QHVCAppendUpload.h
//  QHVCUploadKit
//
//  Created by niezhiqiang on 2019/4/1.
//  Copyright © 2019 qihoo. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

typedef void(^successBlock)( NSDictionary *_Nullable object,NSError * _Nullable error);
typedef void(^failBlock)(NSError * _Nullable error);
typedef void(^failAppendUploadBlock)(NSError * _Nullable error, int bid);//bid 上传的分块索引
typedef void(^progressBlock)(NSProgress * _Nullable progress);

@interface QHVCAppendUploader : NSObject

/**
 初始化上传

 @param parameter @{@"object":@"上传到服务端的文件名"}
 @param success 成功回调
 @param fail 失败回调
 @param token 签名,在服务器计算
 */
- (void)initAppendUpload:(NSDictionary *)parameter success:(successBlock)success fail:(failBlock)fail token:(NSString *)token;

/**
 追加上传

 @param parameter @{//未解释的字段均由initAppendUpload接口返回
 @"host":host,
 @"tid":tid,
 @"bid":@"分片id从0开始",
 @"scid":scid,
 @"uid":uid,
 @"is_last":@"最后一片传1否则传0",
 @"bhash":@"每个分片data的sha1值",
 @"file":NSData类型每个分片的data
 }
 @param success 成功回调
 @param fail 失败回调
 @param uploadProgress 进度回调
 */
- (void)appendUpload:(NSDictionary *)parameter success:(successBlock)success fail:(failAppendUploadBlock)fail progress:(progressBlock _Nullable)uploadProgress;

/**
 完成上传

 @param parameter @{//未解释的字段均由initAppendUpload接口返回
 @"host":host,
 @"object":@"上传到服务端的文件名",
 @"tid":tid,
 @"scid":scid,
 @"uid":uid
 }
 @param success 成功回调
 @param fail 失败回调
 */
- (void)finishAppendUpload:(NSDictionary *)parameter success:(successBlock)success fail:(failBlock)fail;

/**
 取消上传任务
 */
- (void)cancel;

@end

NS_ASSUME_NONNULL_END
